__all__ = ["status"]

from pathlib import Path
from picsi.vendored.get_output import get_output


def status():
    """
    See status
    """

    state_picsi_is_installed = Path("/home/pi/.picsi/state/picsi_is_installed")
    state_firmware_is_patched = Path("/home/pi/.picsi/state/firmware_is_patched")
    state_csicollection_is_up = Path("/var/run/picsi/state/csicollection_is_up")

    if state_picsi_is_installed.exists():
        print("Firmware installed: Yes")
    else:
        print("Firmware installed: No")

    if state_firmware_is_patched.exists():
        print("Firmware enabled: Yes")
    else:
        print("Firmware enabled: No")

    if state_csicollection_is_up.exists():
        chanspec = get_output(["nexutil", "-k"])
        print("CSI collection running: Yes")
        print(f"chanspec: {chanspec}")
    else:
        print("CSI collection running: No")



#!/usr/bin/env python3

import os

# Elevate picsi to root if not running as root
if os.getuid() != 0:
    import sys
    import subprocess
    from picsi.vendored.get_output import get_output

    executable = get_output(["which", "picsi"])
    arguments = sys.argv[1:]

    p = subprocess.run(["sudo", "-E", executable] + sys.argv[1:])

    sys.exit(p.returncode)
else:
    from picsi.vendored.get_output import get_output

    get_output(["chown", "-R", "pi:pi", "/home/pi/.picsi"])


import typer
from click import Group

from picsi.commands.install import install
from picsi.commands.uninstall import uninstall
from picsi.commands.enable import enable
from picsi.commands.disable import disable
from picsi.commands.build import build
from picsi.commands.up import up
from picsi.commands.down import down
from picsi.commands.status import status


# Typer prints commands in the help menu
# in a random order. This fixes that
# https://skeptric.com/typer-command-order/
class NaturalOrderGroup(Group):
    def list_commands(self, ctx):
        return self.commands.keys()


app = typer.Typer(cls=NaturalOrderGroup)

app.command()(install)
app.command()(uninstall)
app.command()(enable)
app.command()(disable)
app.command()(up)
app.command()(down)
app.command()(status)
app.command()(build)

if __name__ == "__main__":
    app()



import subprocess
from halo import Halo
from os import environ
from pathlib import Path
from datetime import datetime
from typing import Any, Union, List


def run_commands(
    # typing.List before py 3.9
    commands: List[Union[str, List[Any]]],
    spinner: Halo = None,
    check_return: bool = True,
    cwd: str = None,
    env: dict = None,
    log_title: str = "picsi",
) -> None:

    osenv = dict(environ)

    if env is not None:
        osenv.update(env)

    Path("/home/pi/.picsi/logs/").mkdir(exist_ok=True, parents=True)

    # fmt: off
    with \
         open("/home/pi/.picsi/logs/cmd.log", "a", buffering=1) as log_cmd, \
         open("/home/pi/.picsi/logs/time.log", "a", buffering=1) as log_time, \
         open("/home/pi/.picsi/logs/stdout.log", "a", buffering=1) as log_stdout, \
         open("/home/pi/.picsi/logs/stderr.log", "a", buffering=1) as log_stderr:
        # fmt: on

        for c in commands:
            if type(c) == str:
                log_cmd.write(f"\n{c}\n")
                log_time.write(f"\n{datetime.now(): %b %d %Y %H:%M:%S} {log_title} [{0:7.2f}] 0: {c}\n")

                head = c.split(" ")[0]
                body = " ".join(c.split(" ")[1:])

                if head in ["#"]:
                    if spinner is not None:
                        spinner.text = body
                    else:
                        print(body)
                elif head in ["cd"]:
                    cwd = body

            elif type(c) == list:
                log_cmd.write(f"{c}\n")

                time_start = datetime.now()

                p = subprocess.run(
                    [str(i) for i in c],
                    stdout=log_stdout,
                    stderr=log_stderr,
                    stdin=subprocess.PIPE,
                    encoding="utf-8",
                    cwd=cwd,
                    env=osenv,
                )
                
                time_stop = datetime.now()

                log_time.write(f"{time_start: %b %d %Y %H:%M:%S} {log_title} [{(time_stop - time_start).total_seconds():7.2f}] {p.returncode}: {c}\n")

                if check_return:
                    p.check_returncode()



import subprocess
from os import environ
from pathlib import Path  # imported for typing
from typing import List, Dict, Any


def get_output(
    command: List[Any],
    cwd: Path = None,
    env: Dict[str, str] = None,
    check_return: bool = True,
) -> str:

    if cwd is not None:
        cwd = cwd.resolve()

    osenv = dict(environ)

    if env is not None:
        osenv.update(env)

    command = [str(c) for c in command]
    p: subprocess.CompletedProcess = subprocess.run(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        stdin=subprocess.PIPE,
        encoding="utf-8",
        cwd=cwd,
        env=osenv,
    )

    if check_return:
        p.check_returncode()

    return p.stdout.strip()



__all__ = ["install"]

import requests
from typer import Exit
from pathlib import Path
from halo import Halo

from picsi.vendored.get_uname import get_uname
from picsi.vendored.run_commands import run_commands


def get_binaries(
    uname_r: str,
    destination: Path,
    variant: str = "base",
    branch: str = "main",
    repository: str = "nexmonster/nexmon_csi_bin",
) -> None:
    url = f"https://github.com/{repository}/raw/{branch}/{variant}/{uname_r}.tar.xz"

    response = requests.get(url)

    if response.status_code != 200:
        print(
            f"""\n
        Error: Couldn't download binaries. http code {response.status_code}

        Pre-compiled binaries are not uploaded for your kernel's version: {uname_r} yet.
        Please create a new Issue on Github and tell us which kernel you are using.

        Meanwhile, you can build Nexmon_csi from source by running `picsi build`,
        and then run `picsi install` again.
        """
        )
        raise Exit(1)

    destination.parent.mkdir(exist_ok=True)

    with open(destination, "wb") as ofile:
        ofile.write(response.content)


def extract_archive(location: Path) -> None:
    run_commands([["tar", "-xvJf", location, "-C", location.parent]])


def install():
    """
    Install Nexmon_CSI from binaries
    """

    # Check if picsi is already installed
    state_picsi_is_installed = Path("/home/pi/.picsi/state/picsi_is_installed")
    state_picsi_is_installed.parent.mkdir(exist_ok=True, parents=True)

    if state_picsi_is_installed.is_file():
        print("Error. Firmware is already installed.")
        raise Exit(10)

    with Halo(spinner="dots") as spinner:

        Path("/home/pi/.picsi/").mkdir(exist_ok=True)

        path_nexmon_csi_bin = Path(f"/home/pi/.picsi/bins/{get_uname('-r')}")
        path_nexmon_csi_bin_tarxz = Path(str(path_nexmon_csi_bin) + ".tar.xz")

        # Read system info
        spinner.text = "Reading system info"
        uname_r = get_uname("-r")

        # Download binaries
        spinner.text = "Downloading binaries"
        if not path_nexmon_csi_bin_tarxz.is_file():
            get_binaries(uname_r, path_nexmon_csi_bin_tarxz)

        # Extract binaries
        spinner.text = "Extracting binaries"
        extract_archive(path_nexmon_csi_bin_tarxz)

        # fmt: off
        run_commands([
            "# Installing Nexutil",
            ["ln", "-s", path_nexmon_csi_bin / "nexutil/nexutil", "/usr/local/bin/nexutil"],

            "# Installing Makecsiparams",
            ["ln", "-s", path_nexmon_csi_bin / "makecsiparams/makecsiparams", "/usr/local/bin/mcp"],
            ["ln", "-s", path_nexmon_csi_bin / "makecsiparams/makecsiparams", "/usr/local/bin/makecsiparams"],

            "# Setting up WiFi",
            ["rfkill", "unblock", "all"],
            ["raspi-config", "nonint", "do_wifi_country", "US"],

            "# Expanding SD card",
            ["raspi-config", "nonint", "do_expand_rootfs"],
        ], spinner, log_title='cmd-install')
        # fmt: on

        state_picsi_is_installed.touch()



from pathlib import Path
import tomli


def ci_read_distlink(path_picsi_ci: Path = Path("/boot/picsi-ci.toml")) -> str:
    # NOTE: /boot/picsi-ci.toml is created by me while
    # flashing the Pi. It would not exist on your setup.

    if path_picsi_ci.exists():
        with open(path_picsi_ci, "rb") as infile:
            return tomli.load(infile)["dist"]["link"]
    else:
        return ""



from pathlib import Path


def ci_read_distdate(path_rpi_issue: Path = Path("/etc/rpi-issue")) -> str:
    # https://raspberrypi.stackexchange.com/questions/91548/how-to-get-the-release-date-of-os-in-raspberry-pi

    with open(path_rpi_issue, "r") as rpi_issue:
        line = rpi_issue.readline()

    return line.strip().split(" ")[-1]



from pathlib import Path
import tomli
from typing import Dict


def ci_read_author(path_picsi_ci: Path = Path("/boot/picsi-ci.toml")) -> Dict[str, str]:
    # NOTE: /boot/picsi-ci.toml is created by me while
    # flashing the Pi. It would not exist on your setup.

    if path_picsi_ci.exists():
        with open(path_picsi_ci, "rb") as infile:
            return tomli.load(infile)["author"]
    else:
        return {
            "name": "Anonymous",
            "email": "anonymous@example.com",
            "website": "https://www.example.com/",
            "license": "Unknown",
        }



__all__ = ["build"]

from halo import Halo
from pathlib import Path

from picsi.vendored.mkmanifest import mkmanifest
from picsi.vendored.run_commands import run_commands
from picsi.vendored.get_output import get_output
from picsi.vendored.get_uname import get_uname
from picsi.vendored.ci_resolve_proxy import ci_resolve_proxy


def build(
    url: str = "https://github.com/seemoo-lab/nexmon_csi",
    branch: str = "master",
    nexmon_url: str = "https://github.com/seemoo-lab/nexmon.git",
    nexmon_branch: str = "master",
):
    """
    Build Nexmon_CSI from source
    """

    Path("/home/pi/.picsi/").mkdir(exist_ok=True)

    path_nexmon = Path("/home/pi/.picsi/nexmon")
    path_nexmon_csi = Path(
        "/home/pi/.picsi/nexmon/patches/bcm43455c0/7_45_189/nexmon_csi"
    )
    path_nexmon_csi_bin = Path(f"/home/pi/.picsi/bins/{get_uname('-r')}")

    nexmon_kversion = ".".join(get_uname("-r").split(".")[:2]) + ".y"

    apt_deps = [
        "automake",
        "bc",
        "bison",
        "flex",
        "gawk",
        "git",
        "libgmp3-dev",
        "libncurses5-dev",
        "libssl-dev",
        "libtool-bin",
        "make",
        "qpdf",
        "raspberrypi-kernel-headers",
        "texinfo",
    ]

    if "Python 3" in get_output(["python", "--version"]):
        # In systems whose default python is python3,
        # python-is-python2 package is needed for Nexmon
        apt_deps.append("python-is-python2")

    with Halo(spinner="dots") as spinner:

        # fmt: off
        run_commands([
            "# Setting up WiFi",
            ["rfkill", "unblock", "all"],
            ["raspi-config", "nonint", "do_wifi_country", "US"],

            "# Expanding SD card",
            ["raspi-config", "nonint", "do_expand_rootfs"],

            "# Installing Dependencies",
            ["apt", "update", "-y"],
            ["apt", "install", "-y"] + apt_deps,
        ], spinner, log_title='cmd-build')
        # fmt: on

        uname_r = get_uname("-r")
        kernel_headers = Path(f"/lib/modules/{uname_r}/build/Kconfig")

        if not kernel_headers.is_file():
            # fmt: off
            run_commands([
                "# Installing Kernel headers",
                ["wget", "https://raw.githubusercontent.com/RPi-Distro/rpi-source/master/rpi-source", "-O", "/usr/local/bin/rpi-source"],
                ["chmod", "+x", "/usr/local/bin/rpi-source"],
                ["rpi-source", "-q", "--tag-update"],
                ["rpi-source"]
            ], spinner, log_title='cmd-build')
            # fmt: on

        # fmt: off
        run_commands([
            "# Backing up original binaries",
            ["mkdir", "-p",
                path_nexmon_csi_bin / "makecsiparams/",
                path_nexmon_csi_bin / "nexutil/",
                path_nexmon_csi_bin / "original/",
                path_nexmon_csi_bin / "patched/",
            ],
            ["cp",
                "/lib/firmware/brcm/brcmfmac43455-sdio.bin",
                path_nexmon_csi_bin / "original/",
            ],
            ["cp",
                get_output(["modinfo", "brcmfmac", "-n"]),
                path_nexmon_csi_bin / "original/",
            ],

            "# Downloading Nexmon",
            ["git", "clone", ci_resolve_proxy(nexmon_url, "git"), path_nexmon],
            "cd " + str(path_nexmon),
            ["git", "checkout", nexmon_branch],

            "# Downloading Nexmon_CSI",
            ["git", "clone", ci_resolve_proxy(url, "git"), path_nexmon_csi],
            "cd " + str(path_nexmon_csi),
            ["git", "checkout", branch],

            "# Building libISL",
            "cd " + str(path_nexmon / "buildtools/isl-0.10/"),
            ["autoreconf", "-f", "-i"],
            [path_nexmon / "buildtools/isl-0.10/configure"],
            ["make"],
            ["make", "install"],
            ["ln",
                "-s", "/usr/local/lib/libisl.so",
                "/usr/lib/arm-linux-gnueabihf/libisl.so.10"
            ],

            "# Building libMPFR",
            "cd " + str(path_nexmon / "buildtools/mpfr-3.1.4/"),
            ["autoreconf", "-f", "-i"],
            [path_nexmon / "buildtools/mpfr-3.1.4/configure"],
            ["make"],
            ["make", "install"],
            ["ln",
                "-s", "/usr/local/lib/libmpfr.so",
                "/usr/lib/arm-linux-gnueabihf/libmpfr.so.4"
            ],
        ], spinner, log_title='cmd-build')
        # fmt: on

        env = {
            "ARCH": "arm",
            "SUBARCH": "arm",
            "KERNEL": "kernel7",
            "HOSTUNAME": get_uname("-s"),
            "PLATFORMUNAME": get_uname("-m"),
            "NEXMON_ROOT": str(path_nexmon),
            "CC": str(
                path_nexmon
                / "buildtools/gcc-arm-none-eabi-5_4-2016q2-linux-armv7l/bin/arm-none-eabi-"
            ),
            "CCPLUGIN": str(path_nexmon / "buildtools/gcc-nexmon-plugin-arm/nexmon.so"),
            "ZLIBFLATE": "zlib-flate -compress",
            "Q": "@",
            "NEXMON_SETUP_ENV": "1",
        }

        # fmt: off
        run_commands([
            "# Extracting UCODE, TemplateRAM, and FlashPatches",
            "cd " + str(path_nexmon),
            ["sudo", "-E", "bash", "-c", "make"],

            "# Building Nexmon_CSI",
            "cd " + str(path_nexmon_csi),
            ["sudo", "-E", "bash", "-c", "make install-firmware"],

            "# Building Makecsiparams",
            "cd " + str(path_nexmon_csi / "utils/makecsiparams"),
            ["sudo", "-E", "bash", "-c", "make"],

            "# Building Nexutil",
            "cd " + str(path_nexmon / "utilities/nexutil/"),
            ["sudo", "-E", "bash", "-c", "make"],

            "# Packaging Binaries",
            ["cp",
                path_nexmon / "utilities/nexutil/nexutil", 
                path_nexmon_csi_bin / "nexutil/"
            ],
            ["cp",
                path_nexmon_csi / "utils/makecsiparams/makecsiparams",
                path_nexmon_csi_bin / "makecsiparams/",
            ],
            ["cp",
                path_nexmon_csi / ("brcmfmac_" + nexmon_kversion + "-nexmon/brcmfmac.ko"),
                path_nexmon_csi_bin / "patched/",
            ],
            ["cp",
                path_nexmon_csi / "brcmfmac43455-sdio.bin",
                path_nexmon_csi_bin / "patched/",
            ],
        ], spinner, env=env, log_title='cmd-build')
        # fmt: on

        # Writes the manifest.toml file
        mkmanifest(
            path_nexmon_csi_bin,
            path_nexmon,
            path_nexmon_csi,
            url,
            branch,
            nexmon_url,
            nexmon_branch,
        )

        # fmt: off
        run_commands([
            # When full path is used for tar, that path is preserved in the
            # archive created.
            "cd " + str((path_nexmon_csi_bin / "..").resolve()),
            ["tar", "-cvJf", f"{get_uname('-r')}.tar.xz", f"{get_uname('-r')}"]
        ], spinner, log_title='cmd-build')
        # fmt: on



__all__ = ["enable"]

from pathlib import Path
from halo import Halo
from typer import Exit
from picsi.vendored.run_commands import run_commands
from picsi.vendored.get_uname import get_uname
from picsi.vendored.get_brcmfmacko import get_brcmfmacko


def enable():
    """
    Enable CSI collection
    """

    # Check if picsi is not installed
    state_picsi_is_installed = Path("/home/pi/.picsi/state/picsi_is_installed")
    state_picsi_is_installed.parent.mkdir(exist_ok=True, parents=True)

    if not state_picsi_is_installed.is_file():
        print("Error. You need to run `picsi install` before you can do this.")
        raise Exit(10)

    # Check if firmware is already enabled
    state_firmware_is_patched = Path("/home/pi/.picsi/state/firmware_is_patched")
    state_firmware_is_patched.parent.mkdir(exist_ok=True, parents=True)

    if state_firmware_is_patched.is_file():
        print("Firmware is already enabled.")
        return

    with Halo(spinner="dots") as spinner:
        spinner.text = "Disabling wpa_supplicant on wlan0"

        path_nexmon_csi_bin = Path(f"/home/pi/.picsi/bins/{get_uname('-r')}")
        path_brcmfmacko: Path = get_brcmfmacko()

        # Block WPA_supplicant on wlan0
        with open("/etc/dhcpcd.conf", "a") as ofile:
            ofile.write(
                "\ndenyinterfaces wlan0\ninterface wlan0\n\tnohook wpa_supplicant\n"
            )

        # fmt: off
        run_commands([
            "# Disabling wpa_supplicant on wlan0",
            ["systemctl", "restart", "wpa_supplicant"],
            ["systemctl", "restart", "dhcpcd"],

            "# Applying firmware patches",
            ["cp", path_nexmon_csi_bin / "patched/brcmfmac.ko", path_brcmfmacko],
            ["cp", path_nexmon_csi_bin / "patched/brcmfmac43455-sdio.bin", "/lib/firmware/brcm/brcmfmac43455-sdio.bin"],
            ["rmmod", "brcmfmac"],
            ["modprobe", "brcmutil"],
            ["insmod", path_brcmfmacko],
            ["depmod", "-a"],
        ], spinner, log_title='cmd-enable')
        # fmt: on

        state_firmware_is_patched.touch()



import time
import hashlib
from pathlib import Path
import tomli_w

from picsi.vendored.get_uname import get_uname
from picsi.vendored.ci_read_author import ci_read_author
from picsi.vendored.ci_read_githash import ci_read_githash
from picsi.vendored.ci_read_distdate import ci_read_distdate
from picsi.vendored.ci_read_distlink import ci_read_distlink


def iterdir(dir_path: Path) -> dict:
    hashes = {}

    for child in dir_path.iterdir():
        if child.is_file():
            with open(child.resolve(), "rb") as infile:
                filehash = hashlib.sha256()
                filehash.update(infile.read())

                hashes[child.name] = {"hash": "", "hashtype": "sha256"}
                hashes[child.name]["hash"] = filehash.hexdigest()

        elif child.is_dir():
            hashes[child.name] = iterdir(child)

    return hashes


def mkmanifest(
    path_nexmon_csi_bin: Path,
    path_nexmon: Path,
    path_nexmon_csi: Path,
    url: str,
    branch: str,
    nexmon_url: str,
    nexmon_branch: str,
) -> None:

    manifest = {
        "author": ci_read_author(),
        "build": {
            "uname_a": get_uname("-a"),
            "uname_r": get_uname("-r"),
            "uname_v": get_uname("-v"),
            "rpios_date": ci_read_distdate(),
            "rpios_link": ci_read_distlink(),
            "timestamp": int(time.time()),
        },
        "nexmon_csi": {
            "repository": url,
            "branch": branch,
            "commit": ci_read_githash(path_nexmon_csi),
        },
        "nexmon": {
            "repository": nexmon_url,
            "branch": nexmon_branch,
            "commit": ci_read_githash(path_nexmon),
        },
        "contents": iterdir(path_nexmon_csi_bin),
    }

    with open(path_nexmon_csi_bin / "manifest.toml", "wb") as outfile:
        tomli_w.dump(manifest, outfile)


# mkmanifest(
#     Path(f"/home/pi/.picsi/bins/{get_uname('-r')}"),
#     Path("/home/pi/.picsi/nexmon"),
#     Path("/home/pi/.picsi/nexmon/patches/bcm43455c0/7_45_189/nexmon_csi"),
#     "https://github.com/seemoo-lab/nexmon_csi",
#     "master",
#     "https://github.com/seemoo-lab/nexmon.git",
#     "master",
# )



__all__ = ["down"]


from pathlib import Path
from halo import Halo
from picsi.vendored.run_commands import run_commands
from picsi.vendored.get_output import get_output


def down():
    """
    Stop CSI collection
    """

    # Check if csi collection is not running
    state_csicollection_is_up = Path("/var/run/picsi/state/csicollection_is_up")
    state_csicollection_is_up.parent.mkdir(exist_ok=True, parents=True)

    if not state_csicollection_is_up.is_file():
        print("CSI collection is not running.")
        return

    with Halo(spinner="dots") as spinner:
        spinner.text = "Stopping CSI collection"

        csiparams = get_output(["makecsiparams", "-e", "0"])

        # fmt: off
        run_commands([
            "# Stopping CSI collection",
            ["nexutil", "-Iwlan0", "-s500", "-b", "-l34", f"-v{csiparams}"],

            "# Removing mon0",
            ["ip", "link", "set", "mon0", "down"],
            ["iw", "dev", "mon0", "del"],

            "# Restarting wlan0",
            ["ip", "link", "set", "dev", "wlan0", "down"],
            ["ip", "link", "set", "dev", "wlan0", "up"],
        ], spinner, log_title='cmd-down')
        # fmt: on

        state_csicollection_is_up.unlink()



from pathlib import Path
import tomli


# I have a git mirror at 10.20.30.41 for nexmon and nexmon_csi
# When creating picsi builds, the repositories are cloned from
# there to speed up cloning.
def ci_resolve_proxy(
    key: str, section: str, ci_file: Path = Path("/boot/picsi-ci.toml")
) -> str:
    if ci_file.is_file():
        with open(ci_file, "rb") as f:
            ci_config = tomli.load(f)

            if section in ci_config["proxy"]:
                if key in ci_config["proxy"][section]:
                    return ci_config["proxy"][section][key]

    return key



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['picsi', 'picsi.commands', 'picsi.vendored']

package_data = \
{'': ['*'], 'picsi': ['scripts/*']}

install_requires = \
['halo>=0.0.31,<0.0.32',
 'requests>=2.27.1,<3.0.0',
 'tomli-w>=1.0.0,<2.0.0',
 'tomli>=2.0.0,<3.0.0',
 'typer[all]>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['picsi = picsi.picsi:app']}

setup_kwargs = {
    'name': 'picsi',
    'version': '0.0.13',
    'description': 'CSI collection on Raspberry Pi',
    'long_description': "# picsi\n\nNexmon CSI utilities for Raspberry Pi\n\n***In development &bull; Not ready for testing yet.***\n\n## Features\n\n- [x] ⚡ **Superfast** installs with pre-compiled binaries\n- [x] ⌛ Compiles from source when binaries are not available\n- [x] 🚀 Easy Start/Stop CSI collection with `picsi up` or `picsi down`\n- [x] ✨ Restore original firmware and connect to WiFi with `picsi disable`\n- [ ] 💾 Saves CSI to .pcap files\n- [ ] 📡 Forward CSI packets to other devices for faster collection\n- [ ] 📁 Manage your CSI collection configuration with Profiles\n\n## Install \n\nOn a Raspberry Pi 3B+ or 4B, run:  \n\n```bash\nsudo apt install python3-pip  # install pip for python3\npip3 install picsi            # install picsi \nsource ~/.profile             # update $PATH\n\npicsi install                 # install nexmon_csi\n```\n\n\n`picsi` will download the appropriate firmware and binaries for\nyour system and install them, or compile from source if they\nare not available pre-compiled.\n\n## Usage\n```bash\npicsi enable                  # enable nexmon_csi\npicsi up                      # Collect CSI on 36/80\n```\n\nThis enables Nexmon_csi, and starts CSI collection on channel 36 with\nbandwidth 80 MHz. You can see the traffic with `tcpdump -i wlan0 dst port 5500`.\n\nMore examples:\n```\npicsi up 149/80               # Collect CSI on 149/80\n\npicsi down                    # Stop CSI collection\npicsi disable                 # Restore original WiFi\n\npicsi status                  # See status\n\npicsi --help                  # See the help page\n```\n\n## Docs\n\nPicsi (pronounced pixie?) is a Python tool to install Nexmon CSI on Raspberry Pis.\nIt needs Python version `>= 3.7`, although using the latest version is recommended.\n\nThe best features of picsi, in my opinion, are:\n\n#### Installing Nexmon CSI from pre-compiled binaries.\n\nCompiling Nexmon_CSI on the Pi takes about an hour, and downloads about 1.5 GB of data.\nAnd it needs your attention for the entire duration because you need to reboot the Pi \nmultiple times, and keep a look out for errors.\n\nPicsi downloads appropriate pre-compiled nexmon_csi firmware and binaries (nexutil, makecsiparams) \nfor your kernel from https://github.com/nexmonster/nexcsi-bin.git (repository not available yet), \nand installs them. If binaries are not available, it installs from source, including automatic \nunattended reboots, and logs errors and progress.\n\n#### Forwards CSI packets to an IP\n\nPicsi can forward CSI packets to a different computer on your network, which is potentially\nfaster than the Pi, and can collect more packets than tcpdump on the Pi can.\n\nBut additionally, an app on your phone/laptop can listen to the packets,\nand plot the CSI in realtime or process it.\n\n#### Profiles!\n\nManage your csi collection configuration in profiles!\n\nwrite\n```toml\n[profiles.CustomProfileName]\n    channel = 36\n    bandwidth = 80\n\n    coremask = 1\n    ssmask = 1\n\n    forward = false\n    forward_ip = '192.168.1.25'\n\n    duration = 30\n\n    macids = ['ab:cd:ef:12:34']\n```\n\nin profiles.toml, and you can start csi collection with\n\n`picsi up CustomProfileName`.\n\nThis collects CSI on channel 36, bandwidth 80 from macids for 30 seconds,\nand forwards that CSI to 192.168.1.25. After 30 seconds, CSI collection is stopped\nand original wifi firmware is restored.\n\nYou can also create a set of profiles, and make picsi loop CSI collection over them.\n\nOnly basic CSI collection via profiles will be added first, and other profile features will\nbe added later.\n",
    'author': 'Aravind Reddy Voggu',
    'author_email': 'zerodividedby0@gmail.com',
    'maintainer': 'Aravind Reddy Voggu',
    'maintainer_email': 'zerodividedby0@gmail.com',
    'url': 'https://github.com/nexmonster/picsi.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



__all__ = ["up"]


from pathlib import Path
from halo import Halo
from typer import Argument, Exit
from picsi.vendored.run_commands import run_commands
from picsi.vendored.get_output import get_output
from picsi.commands.down import down as cmd_down


def up(
    chanspec: str = Argument(default="36/80"),
    macs: str = None,
    byte: str = None,
    delay: int = None,
    coremask: int = 1,
    nssmask: int = 1,
    csiparams: str = None,
):
    """
    Start CSI collection
    """

    # Check if firmware is disabled
    state_firmware_is_patched = Path("/home/pi/.picsi/state/firmware_is_patched")
    state_firmware_is_patched.parent.mkdir(exist_ok=True, parents=True)

    if not state_firmware_is_patched.is_file():
        print("Error. You need to run `picsi enable` before you can do this.")
        raise Exit(10)

    # Check if CSI collection is already running
    state_csicollection_is_up = Path("/var/run/picsi/state/csicollection_is_up")
    state_csicollection_is_up.parent.mkdir(exist_ok=True, parents=True)

    if state_csicollection_is_up.is_file():
        cmd_down()

    with Halo(spinner="dots") as spinner:
        if csiparams is None:
            spinner.text = "Generating CSI Params"
            mcp = [
                "makecsiparams",
                "-c",
                chanspec,
                "-C",
                coremask,
                "-N",
                nssmask,
            ]

            if macs is not None:
                mcp += ["-m", macs]

            if byte is not None:
                mcp += ["-b", byte]

            if delay is not None:
                mcp += ["-d", delay]

            csiparams = get_output(mcp)

            print(f"Channel/Bandwidth: {chanspec}")
            print(f"Csiparams: {csiparams}")

        # fmt: off
        run_commands([
            "# Starting CSI collection",
            ["ip", "link", "set", "dev", "wlan0", "up"],
            ["nexutil", "-Iwlan0", "-s500", "-b", "-l34", f"-v{csiparams}"],

            "# Setting up mon0",
            ["iw", "dev", "wlan0", "interface", "add", "mon0", "type", "monitor"],
            ["ip", "link", "set", "mon0", "up"],
        ], spinner, log_title='cmd-up')
        # fmt: on

        state_csicollection_is_up.touch()



__all__ = ["disable"]

from pathlib import Path
from halo import Halo
from picsi.vendored.run_commands import run_commands
from picsi.vendored.get_uname import get_uname
from picsi.vendored.get_brcmfmacko import get_brcmfmacko
from picsi.commands.down import down as cmd_down


def disable():
    """
    Disable CSI collection
    """

    # Check if firmware is already disabled.
    state_firmware_is_patched = Path("/home/pi/.picsi/state/firmware_is_patched")
    state_firmware_is_patched.parent.mkdir(exist_ok=True, parents=True)

    if not state_firmware_is_patched.is_file():
        print("Firmware is already disabled.")
        return

    # Stop CSI collection if it's running.
    state_csicollection_is_up = Path("/var/run/picsi/state/csicollection_is_up")
    state_csicollection_is_up.parent.mkdir(exist_ok=True, parents=True)

    if state_csicollection_is_up.is_file():
        cmd_down()

    with Halo(spinner="dots") as spinner:
        spinner.text = "Restoring original firmware"

        path_nexmon_csi_bin = Path(f"/home/pi/.picsi/bins/{get_uname('-r')}")
        path_brcmfmacko: Path = get_brcmfmacko()

        # Unblock WPA_supplicant on wlan0
        with open("/etc/dhcpcd.conf", "r") as ifile:
            dhcpcd_conf = ifile.read()

        with open("/etc/dhcpcd.conf", "w") as ofile:
            ofile.write(
                dhcpcd_conf.replace(
                    "\ndenyinterfaces wlan0\ninterface wlan0\n\tnohook wpa_supplicant\n",
                    "",
                )
            )

        # fmt: off
        run_commands([
            "# Restoring original firmware",
            ["cp", path_nexmon_csi_bin / "original/brcmfmac.ko", f"{path_brcmfmacko}"],
            ["cp", path_nexmon_csi_bin / "original/brcmfmac43455-sdio.bin", "/lib/firmware/brcm/brcmfmac43455-sdio.bin"],
            ["rmmod", "brcmfmac"],
            ["modprobe", "brcmutil"],
            ["insmod", path_brcmfmacko],
            ["depmod", "-a"],

            "# Enabling wpa_supplicant on wlan0",
            ["systemctl", "enable", "wpa_supplicant"],
            ["wpa_supplicant", "-B", "-c", "/etc/wpa_supplicant/wpa_supplicant.conf", "-i", "wlan0"],
            ["dhcpcd", "wlan0"],

            "# Restarting wlan0",
            ["ip", "link", "set", "dev", "wlan0", "down"],
            ["ip", "link", "set", "dev", "wlan0", "up"],
        ], spinner, log_title='cmd-disable')
        # fmt: on

        state_firmware_is_patched.unlink()
